#!/usr/bin/python

"""
killerbat.py - flying killer bat
"""

import os
import pygame
from pygame.locals import *
from math import *
import random

import resources

def sign(val):
    if val == 0:
        return 0
    else:
        return int(val/fabs(val))

class BatSprite(pygame.sprite.DirtySprite):
    def __init__(self,size):
        pygame.sprite.DirtySprite.__init__(self)
        self.image = resources.bat_spritesheet
        #self.image.set_colorkey((255,0,255))
        self.rect = self.image.get_rect()
        self.source_rect = pygame.Rect((0,0),(size[0],size[1]))
        self.dirty = 2

class KillerBat(object):
    def __init__(self,x,y,id,h_group):
        self.id = id
        self.x = x
        self.y = y
        self.h_group = h_group
        
        self.heading = 'right'
        self.status = "alive"
        self.speed = 4
        self.attack_range = 256
        self.vx = 0
        self.vy = 0
        self.ax = 0
        self.ay = 0
        
        self.frametype = 'fly'
        self.frame = 0
        self.frame_to = 5
        
        self.homing_in = False
        
        self.size = (resources.bat_spritesheet_coords['left'][0][2], \
            resources.bat_spritesheet_coords['left'][0][3])
        self.sprite = BatSprite(self.size)
    
    def kill(self):
        if self.status == "alive":
            self.status = "dying"
            current_tile = (self.x/16)+self.h_group.h_tiledlayers.map_size[0]*(self.y/16)
            self.h_group.h_tiledlayers.RemoveObj(current_tile,self.id)
            self.frametype = 'death'
            self.frame = 0
            self.vy = 6
    
    def update_alive(self):
        
        init_tile = self.h_group.h_tiledlayers.map_size[0]*(self.y/16)+(self.x/16)
        
        # monster behaviour
        dist2p = pow(self.x-self.h_group.h_player.x,2) + pow(self.y-self.h_group.h_player.y,2)
        if dist2p < self.attack_range*self.attack_range and dist2p > 1*1:
            if fabs(self.x-self.h_group.h_player.x) > 0:
                self.ax = -int((self.x-self.h_group.h_player.x)/fabs(self.x-self.h_group.h_player.x))
            if fabs(self.y-self.h_group.h_player.y) > 0:
                self.ay = -int((self.y-self.h_group.h_player.y)/fabs(self.y-self.h_group.h_player.y))
            if not self.homing_in:
                self.homing_in = True
                resources.soundfx['bat'].play()
            #print "attack player"
        else:
            self.ax = 0
            self.ay = 0
            self.vx = 0
            self.vy = 0
            self.homing_in = False
            #print "relaxing"
        if dist2p <= 10*10 and not self.h_group.h_player.exiting: # bite/kill player
            self.h_group.h_player.kill()
        
        # control
        if fabs(self.ax) > 0:
            self.vx = self.vx + self.ax
        if fabs(self.ay) > 0:
            self.vy = self.vy + self.ay
        self.vx = max(min(self.vx,self.speed),-self.speed)
        self.vy = max(min(self.vy,self.speed),-self.speed)
        self.x = self.x + self.vx
        self.y = self.y + self.vy
        
        # Handle all collisions off solid objects
        (self.x,self.y) = self.h_group.h_tiledlayers.HandleObjectWallCollision((self.x,self.y),(10,10),(self.vx,self.vy))
        
        final_tile = self.h_group.h_tiledlayers.map_size[0]*(self.y/16)+(self.x/16)
        if not init_tile == final_tile:
            self.h_group.h_tiledlayers.UpdateObj(init_tile,final_tile,self.id)
        
        # update animation
        if self.vx > 0:
            self.heading = 'right'
        elif self.vx < 0:
            self.heading = 'left'
        self.frametype = 'fly'
        if self.frame_to == 0:
            self.frame_to = 5
            self.frame = self.frame + 1
            if self.frame > 3:
                self.frame = 0
        self.frame_to = max(0,self.frame_to-1)
    
    def update(self):
        if self.status == 'alive':
            self.update_alive()
        elif self.status == 'dying':
            self.y = self.y + self.vy
            if self.y > (self.h_group.h_tiledlayers.map_size[1]+1)*self.h_group.h_tiledlayers.tilesize:
                self.status = 'dead'
    
    def updatesprite(self):
        if self.status == 'alive' or self.status == 'dying':
            src_coords = resources.bat_spritesheet_coords[self.heading][resources.bat_spriteseq[self.frametype][self.frame]]
            self.sprite.source_rect.x = src_coords[0]
            self.sprite.source_rect.y = src_coords[1]
            (camerax,cameray) = (self.h_group.h_camera.x-self.h_group.h_camera.w_view/2,self.h_group.h_camera.y-self.h_group.h_camera.h_view/2)
            self.sprite.rect.x = int(self.x-self.size[0]/2-camerax)
            self.sprite.rect.y = int(self.y-self.size[1]/2-cameray)
        else:
            self.sprite.rect.x = -10000
            self.sprite.rect.y = -10000

class BatGroup(object):
    def __init__(self,h_tiledlayers,h_camera,h_player):
        self.h_tiledlayers = h_tiledlayers
        self.h_camera = h_camera
        self.h_player = h_player
        self.bats = []
        self.sprite_group = pygame.sprite.LayeredDirty([])
    
    def AddBat(self,tile):
        x = self.h_tiledlayers.tilesize*(tile%self.h_tiledlayers.map_size[0])
        y = self.h_tiledlayers.tilesize*(tile/self.h_tiledlayers.map_size[0])
        id = len(self.bats)+1000
        self.bats.append(KillerBat(x,y,id,self))
        self.sprite_group.add(self.bats[-1].sprite)
    
    def Update(self):
        for bat in self.bats:
            bat.update()
            bat.updatesprite()

